
# TDK-Lambda-client-VS.py
# based on TCP Socket client program
# Communicates with TDK-Lambda GENH 40-19 power supply via TCP sockets on ether
# Ed May Nov 15 2011

import socket
import sys
import string
import time

SDELAY = 1.5	# setling time delay (seconds)
HOST = 'daring.cwi.nl'    # The remote host
HOST = 'pcl12.hep.anl.gov'    # The remote host
HOST = 'diva'    # The remote host
HOST = '169.254.169.112'    # The remote host i.e. the power supply
HOST = '169.254.224.205'    # The remote host i.e. the power supply
PORT = 8003                 # The port as used by the server in power supply
				
#HOST = 'pcl12.hep.anl.gov'	# The remote host when testing
#PORT = 50007			# remote port for testing

s = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
s.connect((HOST, PORT))

# dataCollectionModes:	0	single read
#			1	sweep of voltage
#			2	sweep of current
#			3	fix current sweep time
dataCollectionMode = 1
dataCollectionIndex = 0
# establish software limits on voltage and current
voltageSoftLimit = 40.000	# volts
currentSoftLimit =  4.000	# amps
voltageHardLimit = 10.000
currentHardLimit =  1.000
# open an output file in append mode with no buffering
fo1 = open("readLPS.dat","a",0)
fo2 = open("readLPS.meta","a",0)


print "Issue Identity query to ",HOST
s.send('*IDN?')
data = s.recv(1024)
#print 'Received', repr(data)
sdata = string.replace(data,'\n','')
print 'ID string recieved: ', repr(sdata)

print "Query ON/OFF state of the power supply."
s.send('output:state?')
data = s.recv(1024)
#print 'Received', repr(data)
psState = string.replace(data,'\n','')

print "Set the voltage and current limits"
sVoltageLimitMessage=":voltage "+str(voltageHardLimit)
print "Sleep for ", SDELAY, " (sec) to provide settling time."
time.sleep(SDELAY)
s.send(sVoltageLimitMessage)
sCurrentLimitMessage=":current "+str(currentHardLimit)
s.send(sCurrentLimitMessage)
print "Sleep for ", SDELAY, " (sec) to provide settling time."
time.sleep(SDELAY)


print "Query the voltage and current limits:",
s.send('source:voltage?')
data=s.recv(1024)
#print 'Received', repr(data)
sVoltageLimit=string.replace(data,'\n','')
s.send('source:current?')
data=s.recv(1024)
#print 'Received', repr(data)
sCurrentLimit=string.replace(data,'\n','')
print sVoltageLimit,"(volts)",sCurrentLimit,"(amps)"


if  psState == 'OFF' :
	print "Found Power Supply Output state is OFF,"
	sans=raw_input('Turn on Power Supply (Y/N): ')
	if sans == 'Y' :
	   print "Turning Power Supply Output ON.."
	   s.send('output:state ON')	# does not return any message data
	   s.send('output:state?')    # ask for state to update psState variable
	   data = s.recv(1024)
	   psState = string.replace(data,'\n','')
	else :
	   print "Exit the program..."
	   s.close()
	   fo1.close()
	   fo2.close()
	   sys.exit(1)

print "Query operating mode of the power supply."
print "CV = Constant Voltage CC = Constant Current"
s.send('source:mode?')
data = s.recv(1024)
#print 'Received', repr(data)
sdata = string.replace(data,'\n','')
print "Power Supply operating mode is", sdata

print ""

if psState == 'ON' :
	vNew=0.516
	while vNew > voltageSoftLimit :
	   print "User requested voltage > software limit ",voltageSoftLimit
	   sVoltage=raw_input("Enter new voltage:")
	   vNew=float(sVoltage)
	print "Set voltage to ",vNew," Volts"
	message = ':voltage '+str(vNew)
	s.send(message)
	# note this command does not return any data
	print "Sleep for ", SDELAY, " (sec) to provide settling time."
	time.sleep(SDELAY)
	print "Measure Voltage and Current values:"
	s.send('measure:voltage?')
	data=s.recv(1024)
	sdata = string.replace(data,'\n','')
	print 'Output Voltage = ',sdata," (Volts)"
	s.send('measure:current?')
	data=s.recv(1024)
	sdata = string.replace(data,'\n','')
	print 'Output Current = ',sdata," (Amps)"
	s.send('source:mode?')
	data = s.recv(1024)
	sdata = string.replace(data,'\n','')
	print "Power Supply operating mode is", sdata
else :
	print "Power Supplay Output is OFF."

if psState == 'ON' :
	fo2.write("offset=%f\n" % time.time())
	fo2.write("unset label")
	slabel='set label "'+time.ctime()+'" at graph 0.05,0.5 left'
	fo2.write("%s\n" % slabel)
	# Do a simple IV curve by sweeping the voltage
	dataCollectionMode=1
	dataCollectionIndex=0
	vMin=0.0
	vMax=20.0
	vSteps=20
	iMin=0.0
	iMax=1.0
	iSteps=20
	while vMax > voltageSoftLimit :
		print "User requested max voltage for sweep > software limit "
		,voltageSoftLimit
		sVoltage=raw_input("Enter new voltage:")
		vMax=float(sVoltage)
	deltaVoltage = (vMax - vMin) / vSteps
	vNext = vMin
	print "Sleep for ", SDELAY, " (sec) to provide settling time."
	print "Scan:","sPSmode,voltRead,currentRead,loadResistance,loadPower"
	while vNext < vMax :
		#print "Set voltage to ",vNew," Volts"
		message = ':voltage '+str(vNext)
		s.send(message)
		# note this command does not return any data
		time.sleep(SDELAY)
		#print "Measure Voltage and Current values:"
		s.send('measure:voltage?')
		data=s.recv(1024)
		sdata = string.replace(data,'\n','')
		voltRead = float(sdata)
		s.send('measure:current?')
		data=s.recv(1024)
		sdata = string.replace(data,'\n','')
		currentRead = float(sdata)
		s.send('source:mode?')
		data = s.recv(1024)
		sPSmode = string.replace(data,'\n','')
		try :
			loadResistance = voltRead/currentRead
		except ZeroDivisionError:
			sLoadResistance = '-'
		else :
			sLoadResistance = str(loadResistance)
		loadPower = voltRead*currentRead
		print "Data:",dataCollectionMode,dataCollectionIndex,sPSmode \
		,voltRead,currentRead,sLoadResistance,loadPower
		timeFromEpoch = time.time()
		timeAndDate = time.ctime()
		fo1.write("%d %d %s %f %f %s %f %f %s\n" % (
		dataCollectionMode,dataCollectionIndex,sPSmode, voltRead,
		currentRead,sLoadResistance,loadPower,timeFromEpoch,
		timeAndDate) )
		vNext = vNext + deltaVoltage
		dataCollectionIndex = dataCollectionIndex +1
else :
	print "Power Supplay Output is OFF."

vNew = 0.0
print "At end of program Set voltage to ",vNew," Volts"
message = ':voltage '+str(vNew)
s.send(message)

fo1.close()
fo2.close()
s.close()
sys.exit(0)
